@@ -63,16 +63,30 @@ public:
     void rename(const QUrl& src, const QUrl& dest, KIO::JobFlags flags) override;
     void copy(const QUrl& src, const QUrl& dest, int mode, KIO::JobFlags flags) override;
 
+    void setError(KIO::Error errid, const QString &text);
+    // NFSProtocol should not call these.  See setError() and finishOperation().
+    void finished() = delete;
+    void error(int errid, const QString &text) = delete;
+
+    bool usedirplus3() const					{ return (m_usedirplus3); }
+
 protected:
-    // Verifies the current protocol and connection state, returns true if valid.
-    bool verifyProtocol();
+    // Verifies the URL, current protocol and connection state, returns true if valid.
+    bool verifyProtocol(const QUrl &url);
+    void finishOperation();
 
 private:
     NFSProtocol* m_protocol;
 
-    // We need to cache this because the @openConnection call is responsible
+    // We need to cache these because the @openConnection call is responsible
     // for creating the protocol, and the @setHost call might happen before that.
     QString m_host;
+    QString m_user;
+
+    bool m_usedirplus3;
+
+    KIO::Error m_errorId;
+    QString m_errorText;
 };
 
 
@@ -103,13 +117,8 @@ public:
 
     bool isInvalid() const
     {
-        return m_isInvalid;
+        return m_size==0 && m_linkSize==0;
     }
-    void setInvalid()
-    {
-        m_isInvalid = true;
-    }
-
     bool isLink() const
     {
         return m_isLink;
@@ -134,13 +143,12 @@ protected:
     // Set to the link source's handle.
     char* m_linkHandle;
     unsigned int m_linkSize;
-
-    bool m_isInvalid;
     bool m_isLink;
 };
 
 typedef QMap<QString, NFSFileHandle> NFSFileHandleMap;
 
+
 class NFSProtocol
 {
 public:
@@ -153,7 +161,7 @@ public:
     virtual void openConnection() = 0;
     virtual void closeConnection() = 0;
 
-    virtual void setHost(const QString& host) = 0;
+    virtual void setHost(const QString &host, const QString &user = QString());
 
     virtual void put(const QUrl& url, int _mode, KIO::JobFlags _flags) = 0;
     virtual void get(const QUrl& url) = 0;
@@ -193,17 +201,31 @@ protected:
     bool isValidPath(const QString& path);
     bool isValidLink(const QString& parentDir, const QString& linkDest);
 
-    int openConnection(const QString& host, int prog, int vers, CLIENT*& client, int& sock);
+    KIO::Error openConnection(const QString& host, int prog, int vers, CLIENT*& client, int& sock);
 
     bool checkForError(int clientStat, int nfsStat, const QString& text);
 
     void createVirtualDirEntry(KIO::UDSEntry& entry);
 
+    QString listDirInternal(const QUrl &url);
+    QString statInternal(const QUrl &url);
+    void completeUDSEntry(KIO::UDSEntry &entry, uid_t uid, gid_t gid);
+    void completeInvalidUDSEntry(KIO::UDSEntry &entry);
+
+    QString currentHost() const					{ return (m_currentHost); }
+    NFSSlave *slave() const					{ return (m_slave); }
+    void setError(KIO::Error errid, const QString &text)	{ m_slave->setError(errid, text); }
+
 private:
     NFSSlave* m_slave;
+    QString m_currentHost;
+    QString m_currentUser;
 
     NFSFileHandleMap m_handleCache;
     QStringList m_exportedDirs;
+
+    QHash<uid_t, QString> m_usercache;
+    QHash<gid_t, QString> m_groupcache;
 };
 
 #endif
